using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Utilities;
using Produmex.Foundation.Data.DbClient;
using Produmex.Sbo.Logex.BMW.Generators.BarcodeController;
using Produmex.Sbo.Logex.BMW.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.BMW.Providers;
using Produmex.Sbo.Logex.BMW.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Sbo.Logex.BMW.DataObjects;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// Reception
    /// </summary>
    public class WorkflowScript_ASNReceptionBMWScript : WorkflowInstanceScriptBase
    {
        /// <summary>
        /// Definitions
        /// </summary>
        private static class Definitions
        {
            /// <summary>
            /// The separator for separating strings values
            /// </summary>
            public const string SEPARATOR = ", ";

            /// <summary>
            /// Minimum number of characters when entering reason
            /// </summary>
            public const int MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON = 10;

            /// <summary>
            /// Parent colli number starts with z
            /// </summary>
            public const string PARENT_COLLI_INDICATOR = "A";

            /// <summary>
            /// Child colli number starts with K
            /// </summary>
            public const string CHILD_COLLI_INDICATOR = "K";


            /// <summary>
            /// Child colli of parent number starts with F
            /// </summary>
            public const string CHILD_PARENT_COLLI_INDICATOR = "F";

            /// <summary>
            /// usage field value
            /// </summary>
            public const string USAGE_FIELD = "256";




        }


        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;


        private CultureInfo current_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
        }

        private PmxOseCompany currentPmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
        }


        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        #region Constructors

        public WorkflowScript_ASNReceptionBMWScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {

        }

        #endregion

        #region WorkflowInstanceScriptBase Members

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            IBarcodeController barcodeController = null;

            string initialErrorKey = null;
            string deviceID = session.DeviceInstanceId;
            SboUserInfo userInfo = sboProviderService.GetCurrentUser();
            Message msg = null;
            string query = null;
            bool goBack = false;
            bool isParentColli = false;

            bool askReprint = false;

            int numberOfReports = 0;

            int asnGroup = 0;

            string currentDockCode = null;
            string cardCode = null;
            string invoiceNumber = null;
            string scannedColli = null;
            string masterSSCC = null;

            List<string> receivedCollis = null;
            PmxOseWarehouseInfo whsInfo = null;
            PmxItemInfo itemInfo = null;

            // Dictionary<string, PmxItemInfo> listOfItemInfos = null;

            DataSet dsDocks = null;
            DataSet dsASN = null;
            DataSet dsOpenASNLines = null;
            DataSet dsTemp = null;
            DataSet dsSSCC = GetNewSSCCDataSet();

            KeyValuePair<Collection<DocumentRef>, Collection<int>> listOfGeneratedDocEntry = new KeyValuePair<Collection<DocumentRef>, Collection<int>>();
            Collection<LogisticUnitGoodsReceiptBMW> purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceiptBMW>();
            // List<int?> luids = new List<int?>();
            Collection<Dictionary<string, object>> listOfListOfParameters = new Collection<Dictionary<string, object>>();

            whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);


            bool askUserToPrintLabels = false;


            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
             delegate (PmxDbConnection conn, object[] parameters)
             {
                 PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                 barcodeController = bcProv.GetBarcodeController();

                 PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider();
                 Dictionary<string, PmxParameterValue> pdParams = prov.GetParametersPurchaseOrderGenerator(conn);

                 askUserToPrintLabels = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ASK_TO_PRINT_LABEL, false);

                 return null;
             });



            Step_SelectDock:

            #region SELECT DOCK

            currentDockCode = null;
          

            // Get locations
            query = BuildQuery.GetCodeNameDockOrderByName(false, true, true, whsInfo.PmxWarehouseCode, deviceID);
            dsDocks = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsDocks) == 1 && !goBack)
            {
                currentDockCode = Convert.ToString(dsDocks.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
                //reset warehouse info
                whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), currentDockCode);
                goto Step_OpenASNID;
            }
            goBack = false;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_LOCATION",
                "LocationDS", dsDocks
                ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;
            if (msg.Name.EndsWith(".LocationSelected"))
            {
                currentDockCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                //reset warehouse info
                whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), currentDockCode);

                goto Step_OpenASNID;
            }

            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectDock;
            }

            //back requested
            return;

            #endregion

            Step_OpenASNID:

            #region SELECT OPEN ASN ID

           


            query = BuildQuery.GetOpenAdvanceShippingNoticesForReception(currentDockCode, whsInfo.PmxWarehouseCode, deviceID);
            dsASN = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectSupplierScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", "MSG_TITLE_SELECT_OPEN_ASN",
               "SupplierDS", dsASN,
               "BarcodeController", barcodeController
               ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".SupplierSelected"))
            {

                invoiceNumber = ExtractParameter<string>(msg.Parameters, "supplierCode");


                DataRow[] rows = dsASN.Tables[0].Select(SupplierDataSet.Columns.SUPPLIER_CODE.Name + " = '" + invoiceNumber.ToString() + "'");
                if (rows.Length > 0)
                {
                    asnGroup = Convert.ToInt32(rows[0][PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME]);
                    cardCode = Convert.ToString(rows[0][PmxAdvanceShippingNoticeTable.Columns.UF2.NAME]);

                    //reset open collies 
                    ResetCollis(asnGroup);
                    //sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                    //      delegate (PmxDbConnection conn, object[] parameters)
                    //      {

                    //          PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
                    //          query = BuildQuery.GetAllScannedASNLines(asnGroup, sboProviderService.GetDbTool());
                    //          dsOpenASNLines = sboProviderService.RunView(false, null, null, query);

                    //          foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
                    //          {
                    //              PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
                    //              asnBMW.ReceivedQtyBMW = null;
                    //              prov.UpdateBO(asnBMW, false, false);
                    //          }
                    //          return null;
                    //      }));


                    goto Step_ScannColli;
                }

            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectDock;
            }

            #endregion

            Step_ScannColli:

            #region SCAN COLLI

            scannedColli = null;
            isParentColli = false;
            receivedCollis = new List<string>();
            dsSSCC = GetNewSSCCDataSet();
            dsOpenASNLines = null;


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                  this.DefaultCultureInfo, BuildParamCollection(
                  "InitialErrorKey", initialErrorKey,
                  "TitleKey", "MSG_TITLE_SCAN_ACOLLI_KCOLLI",
                   "BarcodeController", barcodeController,
                   "AllowedButtons", new string[] { "MSG_BUTTON_RECEIVE" },               
                  "AllowToGoBack", true));
            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".IIdentifyGS1DataScreenReply.BackRequested"))
            {

                sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                  delegate (PmxDbConnection conn, object[] parameters)
                  {

                      PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
                      query = BuildQuery.GetAllScannedASNLines(asnGroup, sboProviderService.GetDbTool());
                      dsTemp = sboProviderService.RunView(false, null, null, query);
                                                              
                      return null;
                  }));

                if (dsTemp.Tables[0].Rows.Count > 0)
                {
                    //still open collies
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                      DefaultCultureInfo.Get(), BuildParamCollection(
                                          "TitleKey", "MSG_TITLE_RECEIVE_OPEN_COLLIES",
                                          "MessageKey", "MSG_TITLE_RECEIVE_OPEN_COLLIES"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_MakeReceipt;
                    }
                    else
                    {
                        ResetCollis(asnGroup);
                        //sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                        //   delegate (PmxDbConnection conn, object[] parameters)
                        //   {

                        //       PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
                        //       query = BuildQuery.GetAllScannedASNLines(asnGroup, sboProviderService.GetDbTool());
                        //       dsOpenASNLines = sboProviderService.RunView(false, null, null, query);

                        //       foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
                        //       {
                        //           PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
                        //           asnBMW.ReceivedQtyBMW = null;
                        //           prov.UpdateBO(asnBMW, false, false);
                        //       }
                        //       return null;
                        //   }));
                    }

                    goto Step_OpenASNID;

                }
                goto Step_OpenASNID;
            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                goto Step_MakeReceipt;
            }

            //if (msg.Name.EndsWith(".IIdentifyGS1DataScreenReply.DataEntered"))
            //{
            //    scannedColli = ExtractParameter<string>(msg.Parameters, "stringValue");
            //    goto Step_ValidateColli;
            //}

            if (msg.Name.EndsWith(".DataEntered"))
            {
                BarcodeContainer barcodeContainerLUID = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

                //check what has been scanned
                //if (barcodeContainerLUID.GS1DataObject != null && !string.IsNullOrEmpty(barcodeContainerLUID.GS1DataObject.Sscc))
                //{
                    //user scanned an SSCC, so we check if it is allowed
                    scannedColli = barcodeContainerLUID.GS1DataObject.Sscc;
                    goto Step_ValidateColli;
                    
                //}       
               

            }

            goBack = true;
            goto Step_OpenASNID;

            #endregion

            Step_ValidateColli:

            #region VALIDATE COLLI

            //restrict to K an A numbers
            if (!(scannedColli.StartsWith(Definitions.PARENT_COLLI_INDICATOR) || scannedColli.StartsWith(Definitions.CHILD_COLLI_INDICATOR)))
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                     this.DefaultCultureInfo, BuildParamCollection(
                     "MessageKey", "MSG_ERROR_COLLI_NOT_VALID",
                     "ShowButton", true
                     ));
                msg = WaitForMessage();
                goto Step_ScannColli;
            }


            //get colli
            if (scannedColli.StartsWith(Definitions.PARENT_COLLI_INDICATOR))
            {
                isParentColli = true;
            }

            else
            {
                isParentColli = false;
            }

            receivedCollis.Add(scannedColli);
            query = BuildQuery.GetAllOpenASNLinesToReceive(asnGroup, receivedCollis, isParentColli, sboProviderService.GetDbTool());



            dsOpenASNLines = sboProviderService.RunView(false, null, null, query);

            //Valid colli number?
            if (dsOpenASNLines.Tables[0].Rows.Count == 0)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                     this.DefaultCultureInfo, BuildParamCollection(
                     "MessageKey", "MSG_ERROR_COLLI_NOT_VALID",
                     "ShowButton", true
                     ));
                msg = WaitForMessage();
                goto Step_ScannColli;
            }


            if (scannedColli.StartsWith(Definitions.PARENT_COLLI_INDICATOR))
            {
                masterSSCC = dsOpenASNLines.Tables[0].Rows.Count > 0 ? dsOpenASNLines.Tables[0].Rows[0][PmxAdvanceShippingNoticeBMWTable.Columns.MasterSSCC.NAME].ToString() : null;
                isParentColli = true;
                dsSSCC = GetNewSSCCDataSet();
                goto Step_ScannParentColli;
            }

            else
            {
                //Already scanned
                if (!string.IsNullOrEmpty(dsOpenASNLines.Tables[0].Rows[0][PmxAdvanceShippingNoticeBMWTable.Columns.BMW_receivedQuantity.NAME].ToString()))
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                          this.DefaultCultureInfo, BuildParamCollection(
                          "MessageKey", "MSG_ERROR_COLLI_ALREADY_SCANNED",
                          "ShowButton", true
                          ));
                    msg = WaitForMessage();
                    goto Step_ScannColli;
                }

                if (scannedColli.StartsWith(Definitions.CHILD_PARENT_COLLI_INDICATOR))
                {
                    masterSSCC = dsOpenASNLines.Tables[0].Rows.Count > 0 ? dsOpenASNLines.Tables[0].Rows[0][PmxAdvanceShippingNoticeBMWTable.Columns.MasterSSCC.NAME].ToString() : null;
                    dsSSCC = GetNewSSCCDataSet();
                    isParentColli = true;
                    goto Step_ScannParentColli;
                }

                // asnId = Convert.ToInt32(dsOpenASNLines.Tables[0].Rows[0][PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]);
                goto Step_ReceiveColli;
            }





            #endregion

            Step_ScannParentColli:
            #region RECEIVE PARENT COLLI

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyMultipleSSCCScreen),
           DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", string.Format(base.ResourceStore.GetValue<string>("MSG_TITLE_IDENTIFY_CHILD_COLLI",DefaultCultureInfo.Get() ) , scannedColli),
               "BarcodeController", barcodeController,
               "DisableDeleteButton", false,
               "ForceDataEntry", false,
               "IsNoGS1Data", true,
               "SSCCDS", dsSSCC));
            msg = WaitForMessage();

            //clear the error
            initialErrorKey = null;

            if (msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyMultipleSSCCScreenReply.BackRequested")
            {

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                       "TitleKey", "MSG_TITLE_CHILDCOLLI_NOT_REGISTERED",
                       "MessageKey", "MSG_TITLE_CHILDCOLLI_NOT_REGISTERED"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_ScannColli;
                }
                else
                {
                    goto Step_ScannParentColli;
                }



            }
            if (msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyMultipleSSCCScreenReply.SSCCScanned")
            {
                string sscc = ExtractParameter<string>(msg.Parameters, "sscc");


                DataRow[] rows = dsSSCC.Tables[0].Select(string.Format("{0} = '{1}'", SSCCDataSet.Columns.SSCC.Name, sscc));
                if (rows.Length > 0)
                {
                    initialErrorKey = "MSG_ERROR_SSCC_ALREADY_SCANNED";
                }
                else
                {


                    query = BuildQuery.GetAllOpenASNLinesToReceive(asnGroup, new List<string>() { sscc }, false, sboProviderService.GetDbTool());
                    dsOpenASNLines = sboProviderService.RunView(false, null, null, query);
                    if (dsOpenASNLines.Tables[0].Rows.Count == 0)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                             this.DefaultCultureInfo, BuildParamCollection(
                             "MessageKey", "MSG_ERROR_COLLI_NOT_VALID",
                             "ShowButton", true
                             ));
                        msg = WaitForMessage();
                        goto Step_ScannParentColli;
                    }
                    if (!string.IsNullOrEmpty(dsOpenASNLines.Tables[0].Rows[0][PmxAdvanceShippingNoticeBMWTable.Columns.BMW_receivedQuantity.NAME].ToString()))
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                              this.DefaultCultureInfo, BuildParamCollection(
                              "MessageKey", "MSG_ERROR_COLLI_ALREADY_SCANNED",
                              "ShowButton", true
                              ));
                        msg = WaitForMessage();
                        goto Step_ScannParentColli;
                    }


                    //get the data for the sscc


                    DataRow row = dsSSCC.Tables[0].NewRow();
                    row[SSCCDataSet.Columns.SSCC.Name] = sscc;
                    row[SSCCDataSet.Columns.LUID.Name] = dsOpenASNLines.Tables[0].Rows[0][PmxAdvanceShippingNoticeBMWTable.Columns.LogUnitIdentKey.NAME];



                    dsSSCC.Tables[0].Rows.Add(row);


                    receivedCollis.Add(sscc);


                }

            }

            if (msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyMultipleSSCCScreenReply.DeleteSSCC")
            {
                int luid = ExtractParameter<int>(msg.Parameters, "luid");
                DataRow[] rows = dsSSCC.Tables[0].Select(SSCCDataSet.Columns.LUID.Name + "=" + luid);
                foreach (DataRow row in rows)
                {
                    dsSSCC.Tables[0].Rows.Remove(row);
                }
                dsSSCC.AcceptChanges();

                goto Step_ScannParentColli;
            }

            if (msg.Name.EndsWith("ForwardRequested"))
            {

                query = BuildQuery.GetAllOpenASNLinesToReceive(asnGroup, receivedCollis, false, sboProviderService.GetDbTool());
                dsOpenASNLines = sboProviderService.RunView(false, null, null, query);
                goto Step_ReceiveColli;
            }

            goto Step_ScannParentColli;

            #endregion

            Step_ReceiveColli:

            #region RECEIVE COLLI        

            //RECEIVE 
            try
            {
                sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                   delegate (PmxDbConnection conn, object[] parameters)
                   {

                       PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);

                       foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
                       {
                           PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
                           asnBMW.ReceivedQtyBMW = asnBMW.Quantity;
                           asnBMW.UnitPriceBMW = asnBMW.UnitPriceBMW;
                           prov.UpdateBO(asnBMW, false, false);
                       }

                       return null;
                   }));

                foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
                {
                    //Use the first logistic unit. This because the code at this moment only supports
                    //reception of one logistic unit that must be saved as a good receipt.
                    LogisticUnitItemGoodReceiptBMW logUnitItem = new LogisticUnitItemGoodReceiptBMW();
                    LogisticUnitGoodsReceiptBMW logUnit = new LogisticUnitGoodsReceiptBMW();



                    logUnit.LogisticUnitId = Convertor.ConvertToInt32Nullable(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME]);

                    logUnit.MasterLUID = Convertor.ConvertToInt32Nullable(row[PmxAdvanceShippingNoticeTable.Columns.MasterLogUnitIdentKey.NAME]);

                    logUnit.SSCC = string.IsNullOrEmpty(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString()) ?
                                    null : row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString();
                    logUnit.SupplierPalletNumber = string.IsNullOrEmpty(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString()) ?
                                    null : row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString();

                    logUnitItem.ItemCode = row[PmxAdvanceShippingNoticeTable.Columns.ItemCode.NAME].ToString();
                    logUnitItem.ItriKey = Convertor.ConvertToInt32Nullable(row[PmxAdvanceShippingNoticeTable.Columns.ItemTransactionalInfoKey.NAME]);


                    ////get the iteminfo
                    itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), logUnitItem.ItemCode);

                    logUnit.AllowReceptionWithoutLUID = !itemInfo.CreateSsccOnReception;// && !logUnit.LogisticUnitId.HasValue;

                    logUnitItem.Usage = Definitions.USAGE_FIELD;

                    logUnitItem.UnitPrice = Convertor.ConvertToDoubleNullable(row[PmxAdvanceShippingNoticeBMWTable.Columns.BMW_unitPrice.NAME]);

                    logUnitItem.Quantity = Convert.ToDouble(row[ProductDataSet.Columns.QUANTITY.Name]);
                    //logUnitItem.PurchaseDocRef = new DocumentRef(row[PmxPurchaseOrderLineTable.Columns.ObjectType.NAME].ToString(),
                    //                                    Convert.ToInt32(row[PmxPurchaseOrderLineTable.Columns.DocEntry.NAME]));
                    //logUnitItem.PurchaseDocumentLineNum = Convert.ToInt32(row[ProductDataSet.Columns.LINE_NUMBER.Name]);

                    logUnitItem.BatchNumber1 = row[ProductDataSet.Columns.BATCH1.Name].ToString();
                    logUnitItem.BatchNumber2 = row[ProductDataSet.Columns.BATCH2.Name].ToString();
                    logUnit.ColliNumberBMW = logUnit.SSCC;
                    logUnit.InvoiceBMW = row[PmxAdvanceShippingNoticeTable.Columns.UF1.NAME].ToString();
                    logUnit.ContainerBMW = row[PmxAdvanceShippingNoticeTable.Columns.UF3.NAME].ToString();

                    if (!string.IsNullOrEmpty(row[ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString()))
                    {
                        logUnitItem.BestBeforeDate = DateTime.ParseExact(row[ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString(), "yyyyMMdd", null);
                    }

                    //Instead of adding the logUnitItem we set it like this 
                    //so go back in select location is possible
                    //If we want multiple LogUnitItems then go back on Select Location must not be possible
                    if (logUnit.ItemsOnLogisticUnit.Count.Equals(0))
                    {
                        logUnit.ItemsOnLogisticUnit.Add(logUnitItem);
                    }
                    else
                    {
                        logUnit.ItemsOnLogisticUnit[0] = logUnitItem;
                    }

                    //   logUnit.usage=
                    purchaseDeliveryNoteList.Add(logUnit);
                }

            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //ask user if he wants to try again after error
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_ReceiveColli;
                }

                goto Step_ScannColli;
            }


            //CHECK IF THERE ARE STILL OPEN ITEMS

            //if (isParentColli)
            //{
            //    query = BuildQuery.GetAllOpenASNLines(asnGroup, masterSSCC, sboProviderService.GetDbTool());
            //    dsOpenASNLines = sboProviderService.RunView(false, null, null, query);

            //    if (this.GetNumberOfRows(dsOpenASNLines) > 0)
            //    {
            //        goto Step_ScannParentColli;
            //       
            //    }

            //}

            query = BuildQuery.GetAllOpenASNLines(asnGroup, null, sboProviderService.GetDbTool());
            dsOpenASNLines = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsOpenASNLines) > 0)
            {

                goto Step_ScannColli;

            }


            goto Step_MakeReceipt;



        #endregion

        Step_MakeReceipt:

            #region MAKE RECEIPT       

            dsOpenASNLines = null;


            try
            {
                listOfGeneratedDocEntry = sboProviderService.InvokeMethodWithDbConnection<KeyValuePair<Collection<DocumentRef>, Collection<int>>>(true, false, null, null, new DelegateWithPmxDbConnection<KeyValuePair<Collection<DocumentRef>, Collection<int>>>(
                delegate (PmxDbConnection conn, object[] parameters)
                {
                    PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider((SboConnection)conn);
                    return prov.AddPurchaseDeliveryNoteListOfLogisticUnitsForASNReception(cardCode, currentDockCode, asnGroup, purchaseDeliveryNoteList);
                   

                }));
                purchaseDeliveryNoteList.Clear();

                goto Step_PrintDeliveryDocument;
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                if (listOfGeneratedDocEntry.Key == null)
                {
                    ResetCollis(asnGroup);

                   //sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                   //delegate (PmxDbConnection conn, object[] parameters)
                   //{

                   //    PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
                   //    query = BuildQuery.GetAllScannedASNLines(asnGroup, sboProviderService.GetDbTool());
                   //    dsOpenASNLines = sboProviderService.RunView(false, null, null, query);

                   //    foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
                   //    {
                   //        PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
                   //        asnBMW.ReceivedQtyBMW = null;                        
                   //        prov.UpdateBO(asnBMW, false, false);
                   //    }
                   //    return null;
                   //}));
                }
            }

             

        
    

          

            #endregion

           Step_PrintDeliveryDocument:

            #region PRINT DELIVERY DOCUMENT

            numberOfReports = 0;
            try
            {
                foreach (DocumentRef docRef in listOfGeneratedDocEntry.Key)
                {
                    //trigger event
                    numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
                         delegate (PmxDbConnection conn, object[] parameters2)
                         {
                             PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                             return printProv.PrintReportsForEvent(PmxPrintReportEventType.ReceptionPurchaseDeliveryNoteCreated,
                             currentDockCode, deviceID, docRef.DocEntry, out askReprint);
                         });
                }

            }
            catch (Produmex.Foundation.ProdumexException ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", ex.Error,
                        "ShowButton", true));

                msg = WaitForMessage();
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                        "ShowButton", true));

                msg = WaitForMessage();
            }

            goto Step_DocumentsPrinted;

            #endregion


            Step_DocumentsPrinted:

            #region DOCUMENTS PRINTED

            if (numberOfReports > 0 && askReprint)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT",
                        "TitleKey", "MSG_TITLE_REPRINT_DOCUMENT"));

                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_PrintDeliveryDocument;
                }
            }
            goto Step_OpenASNID;

            #endregion

          
        }

        #endregion

        #region HELPERS



        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }


        /// <summary>
        /// Executes the report Pallet Label.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static object ReceiveCollis(PmxDbConnection conn, object[] parameters)
        {
            PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
            DataSet ds = (DataSet) parameters[0];
            foreach (DataRow row in ds.Tables[0].Rows)
            {
                PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
                asnBMW.ReceivedQtyBMW = asnBMW.Quantity;
                asnBMW.UnitPriceBMW = asnBMW.UnitPriceBMW;
                prov.UpdateBO(asnBMW, false, false);
            }

            return null;
        }

        private object ResetCollis( int? asnGroup)
        {
            String query = null;
            DataSet ds = new DataSet();
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                  delegate (PmxDbConnection conn, object[] parameters)
                  {

                      PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
                      query = BuildQuery.GetAllScannedASNLines(asnGroup, sboProviderService.GetDbTool());
                      ds = sboProviderService.RunView(false, null, null, query);

                      foreach (DataRow row in ds.Tables[0].Rows)
                      {
                          PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
                          asnBMW.ReceivedQtyBMW = null;
                          prov.UpdateBO(asnBMW, false, false);
                      }
                      return null;
                  }));

            return null;
            //PmxAdvanceShippingNoticeBMWProvider prov = new PmxAdvanceShippingNoticeBMWProvider(conn);
            //query = BuildQuery.GetAllScannedASNLines(asnGroup, conn.DbTool());
            //ds = sboProviderService.RunView(false, null, null, query);

            //foreach (DataRow row in ds.Tables[0].Rows)
            //{
            //    PmxAdvanceShippingNoticeBMW asnBMW = prov.GetBO(Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME]));
            //    asnBMW.ReceivedQtyBMW = null;
            //    prov.UpdateBO(asnBMW, false, false);
            //}
            //return null;

        }

        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get item infos.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static Dictionary<string, PmxItemInfo> ExecuteGetItemInfos(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfoInDictionary((Collection<string>)parameters[0]);
        }



        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Gets the new SSCC data set.
        /// </summary>
        /// <returns></returns>
        private DataSet GetNewSSCCDataSet()
        {
            DataSet ds = new DataSet();
            ds.Tables.Add(SSCCDataSet.NAME);
            ds.Tables[0].Columns.Add(SSCCDataSet.Columns.LUID.Name, SSCCDataSet.Columns.LUID.Types[1]);
            ds.Tables[0].Columns.Add(SSCCDataSet.Columns.SSCC.Name, SSCCDataSet.Columns.SSCC.Types[0]);
            ds.Tables[0].Columns.Add(SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Types[0]);
            ds.Tables[0].Columns.Add(SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Types[0]);
            ds.Tables[0].Columns.Add(SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Types[0]);

            return ds;

        }

 

        #endregion

        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {


            private static SboBooleanConvertor m_sboBooleanConvertor = null;
            private static SboBooleanConvertor BooleanConvertor
            {
                get
                {
                    if (m_sboBooleanConvertor == null)
                    {
                        m_sboBooleanConvertor = new SboBooleanConvertor();
                    }
                    return m_sboBooleanConvertor;
                }
            }

            /// <summary>
            /// Gets the name and the code of the  location, ordered by name.
            /// </summary>
            /// <param name="isLoading">Is the dock for loading? When null, no check will be done on this value</param>
            /// <param name="isUnloading">Is the dock for unloading? When null, no check will be done on this value</param>
            /// <param name="useAnd">if set to <c>true</c> the operand between Loading and Unloading is 'AND'.
            /// Otherwise it is 'OR'.</param>
            /// <param name="pmxWarehouseCode">The PMX warehouse code.</param>
            /// <param name="deviceID">The device ID.</param>
            /// <returns>The sql statement to get the dock</returns>
            public static string GetCodeNameDockOrderByName(bool? isLoading, bool? isUnloading, bool useAnd, string pmxWarehouseCode, string deviceID)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));

                //inner join PmxOseDock - PmxOse
                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

                //inner join PmxOseDock - PmxOseStorageLocation
                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                strBuilder.Append("\r\n WHERE ");

                //Dock must be active
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                if (!String.IsNullOrEmpty(pmxWarehouseCode) || !String.IsNullOrEmpty(deviceID))
                {
                    strBuilder.Append(" AND ( ");
                    if (!String.IsNullOrEmpty(pmxWarehouseCode))
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWarehouseCode, CultureInfo.InvariantCulture));
                    }
                    if (!String.IsNullOrEmpty(deviceID))
                    {
                        if (!String.IsNullOrEmpty(pmxWarehouseCode)) strBuilder.Append(" OR ");

                        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                            SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                    }
                    strBuilder.Append(" )");
                }

                if (isUnloading.HasValue || isLoading.HasValue)
                {
                    strBuilder.Append(" AND (");

                    if (isUnloading.HasValue)
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.IsUnloadingDock.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(BooleanConvertor.ConvertBOPropertyToColumn(isUnloading.Value), CultureInfo.InvariantCulture));
                    }

                    //if (isLoading.HasValue)
                    //{
                    //    if (isUnloading.HasValue)
                    //    {
                    //        strBuilder.Append(useAnd ? " AND " : " OR ");
                    //    }

                    //    SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.IsLoadingDock.NAME);
                    //    strBuilder.Append(" = ");
                    //    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(BooleanConvertor.ConvertBOPropertyToColumn(isLoading.Value), CultureInfo.InvariantCulture));
                    //}

                    strBuilder.Append(" )");
                }

                strBuilder.Append("\r\n ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME);

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the open advance shipping notices for reception.
            /// </summary>
            /// <param name="dockCode">The dock code.</param>
            /// <param name="whsCode">The PMX warehouse code.</param>
            /// <param name="deviceID">The device unique identifier.</param>
            /// <returns></returns>
            internal static string GetOpenAdvanceShippingNoticesForReception(string dockCode, string whsCode, string deviceID)
            {
                StringBuilder strBuilder = new StringBuilder();
               

                strBuilder.Append(" SELECT DISTINCT ");
                SqlCommandHelper.AddColumnNameWithAs(strBuilder, PmxAdvanceShippingNoticeTable.Columns.UF1.NAME, SupplierDataSet.Columns.SUPPLIER_CODE.Name);
                  

                strBuilder.Append(", ");
                SqlCommandHelper.AddColumnNameWithAs(strBuilder, PmxAdvanceShippingNoticeTable.Columns.UF1.NAME, SupplierDataSet.Columns.SUPPLIER_NAME.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddColumnNameWithAs(strBuilder, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddColumnNameWithAs(strBuilder, PmxAdvanceShippingNoticeTable.Columns.UF2.NAME, PmxAdvanceShippingNoticeTable.Columns.UF2.NAME);

                strBuilder.AppendFormat("\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxAdvanceShippingNoticeTable.NAME));
                
                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LineStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                strBuilder.AppendFormat("\r\n GROUP BY {0}, {1}, {2}",
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UF1.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UF2.NAME)

                    );


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all open asn lines automatic receive.
            /// </summary>
            /// <param name="asnGroup">The asn unique identifier.</param>
            /// <param name="scannedColli">The colli.</param>
            /// <param name="parentColli">Is a parentColli.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            internal static string GetAllOpenASNLinesToReceive( int asnGroup, List<string> scannedColli, bool parentColli, DbTool dbTool )
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.AppendFormat("SELECT {0}, {1}, {2}, {3}, {4}, {5}, {6}, {7}, {8}, {9}, {10}, {11}, {12}, {13}, {14},{15},{16}",

                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME, ProductDataSet.Columns.INTERNAL_KEY.Name),
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxAdvanceShippingNoticeTable.Columns.ItemCode.NAME)
                         ),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.Quantity.NAME, ProductDataSet.Columns.QUANTITY.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.NAME, PmxItemTable.Columns.InventoryUom.NAME, ProductDataSet.Columns.UOM.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ProductDataSet.Columns.BATCH1.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ProductDataSet.Columns.BATCH2.Name),

                    dbTool.DbServerType == DbServerType.HANA ?
                    string.Format("TO_VARCHAR ({0}, 'YYYYMMDD') AS {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.BEST_BEFORE_DATE.Name)
                        ) :
                    string.Format("CONVERT(NVARCHAR(255), {0}, 112) AS {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.BEST_BEFORE_DATE.Name)
                        ),
             
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.MasterLogUnitIdentKey.NAME, PmxAdvanceShippingNoticeTable.Columns.MasterLogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UF2.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UF1.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UF3.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeBMWTable.Columns.BMW_receivedQuantity.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeBMWTable.Columns.BMW_unitPrice.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeBMWTable.Columns.MasterSSCC.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeBMWTable.NAME, PmxAdvanceShippingNoticeBMWTable.Columns.ItemTransactionalInfoKey.NAME)
                      );
                   
            
                strBuilder.AppendFormat("\r\n FROM {0}",
				   SqlCommandHelper.EscapeTableName(PmxAdvanceShippingNoticeTable.NAME));

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} ",
                  SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.ItemTransactionalInfoKey.NAME)
                  );

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} ",
                SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME)
                );


                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
              SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
				  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
				  SqlCommandHelper.FormatAndEscapeColumnValue(asnGroup, CultureInfo.InvariantCulture));

                if (parentColli)
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.MasterSSCC.NAME));
                    strBuilder.Append(" IN (");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(scannedColli, dbTool.CultureInfo));
                    strBuilder.Append(")");
                }
                else
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.SSCC.NAME));
                    strBuilder.Append(" IN (");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(scannedColli, dbTool.CultureInfo));
                    strBuilder.Append(")");
                }
			

				strBuilder.AppendFormat("\r\n AND {0} = {1}",
				 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LineStatus.NAME),
				 SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append("\r\n AND ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
			}

            /// <summary>
            /// Gets all open asn lines automatic receive.
            /// </summary>
            /// <param name="asnGroup">The asn unique identifier.</param>
            /// <param name="masterSSCC">The mastersscc.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            internal static string GetAllOpenASNLines(int asnGroup, string masterSSCC, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append(" SELECT * ");
                strBuilder.AppendFormat("\r\n FROM {0}",
                   SqlCommandHelper.EscapeTableName(PmxAdvanceShippingNoticeTable.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
                  SqlCommandHelper.FormatAndEscapeColumnValue(asnGroup, CultureInfo.InvariantCulture));        

                strBuilder.AppendFormat("\r\n AND {0} = {1}",
                 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LineStatus.NAME),
                 SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));


                strBuilder.AppendFormat("\r\n AND {0} IS NULL",
                 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeBMWTable.Columns.BMW_receivedQuantity.NAME));
                

                if (!string.IsNullOrEmpty(masterSSCC))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.MasterSSCC.NAME));
                    strBuilder.Append(" IN (");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(masterSSCC, dbTool.CultureInfo));
                    strBuilder.Append(")");
                };



                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets all open asn lines automatic receive.
            /// </summary>
            /// <param name="asnGroup">The asn unique identifier.</param>
            /// <param name="masterSSCC">The mastersscc.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            internal static string GetAllScannedASNLines(int? asnGroup,  DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append(" SELECT * ");
                strBuilder.AppendFormat("\r\n FROM {0}",
                   SqlCommandHelper.EscapeTableName(PmxAdvanceShippingNoticeTable.NAME));

                

               
                     strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LineStatus.NAME),
                 SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                if (asnGroup.HasValue)
                {
                    strBuilder.AppendFormat("\r\n AND {0} = {1}",
                  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
                  SqlCommandHelper.FormatAndEscapeColumnValue(asnGroup, CultureInfo.InvariantCulture));
                }
                strBuilder.AppendFormat("\r\n AND {0} IS not NULL",
                 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeBMWTable.Columns.BMW_receivedQuantity.NAME));


             

                return strBuilder.ToString();
            }


            public static string GetSSCCInfo(string sscc)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT DISTINCT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name);
                strBuilder.Append(',');
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, SSCCDataSet.Columns.LUID.Name);
                strBuilder.Append(',');
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name);

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));


                strBuilder.AppendFormat("FROM {0} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));


                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));


                strBuilder.AppendFormat("WHERE {0}.{1} = {2} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME), SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME), SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }
        }


        #endregion

    }


}
